%%  Author: Panos N. Alevizos
%
%   Ph.D. Student and Research Assistant
%   School of Electrical and Computer Engineering
%   Technical University of Crete
%   Email: palevizos@isc.tuc.gr or bigpan88@hotmail.com
%   Webpage: http://users.isc.tuc.gr/~palevizos/
%
%   Date: January 2018
%
%   References:
%   P. N. Alevizos, K. Tountas, and A. Bletsas, "Multistatic Scatter Radio Sensor Networks for Extended Coverage," IEEE Transactions On Wireless Communications,
%   Volume: XX, Issue: XX, pp. XX--XX, 2018.
%
%   **Inquiries regarding the script provided below are cordially welcome.
%   In case you spot a bug, please let me know.
%   If you use some piece of code for your own work, please cite the
%   article above.**
%
%% this code is for Fig. 7 of the paper

%%
close all;
clear all;
clc;




%%  Propagation and channel variance and noise Parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%noise figure (dB)
noiseFigure = 5;
%noise energy (dBm/Hz)
N0_dB = -174 + noiseFigure;
%Noise energy (Watt/hz)
N0 = 0.001*10^(N0_dB/10);
%F carier
F_carrier = 0.868 * 10^9;
%wavelength (meters)
lambda = 3*10^8 / F_carrier;
%reference distance
d_ref = 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%              Tag Parameters.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Tag data rate (bps).
rate = 1e3;
%bit duration
T = 1/rate;
%Tag scattering efficiency.
scatter_eff = 1e-1;
%antenna load values
Gamma_0 = 1;
Gamma_1 = -1;
Gamma_0_1 = abs(Gamma_0 - Gamma_1);
%number of tags
N = 8
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




%%             Emitter Parameters.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%number of emitters (and slots)
L =  4;
%Emitter transmit power (Watts).
CE_Tx_power_Watts = 0.001 * 10^(35/10)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%             SDR reader Parameters.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SDR reader transmit power (Watts).
SDRReader_Tx_power_Watts =  0.001* 10^(35/10)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





%%     topology parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Grid parameters.
M = 2.5;                  % axis length.
Delta = 0.125;               % Grid resolution (m).
K = round(M/Delta) + 1;     %number of grid points per dimention

%grid 2 x 2
grid_2D = zeros(2, K^2);
grid_2D(1,:) = kron(ones(1,K), [0:K-1])*Delta;
grid_2D(2,:) = kron([0:K-1], ones(1,K))*Delta;


%indexes of position with respect to grid_2D matrix
%SDR reader position index
idx_pos_SDRReader = round(K^2/2);
%carrier emitters position index
idx_pos_CE= [ round(K^2/4+1) , round(K^2/4 + K/2), round(3*K^2 / 4 ), ...
    round(3*K^2 / 4 - K/2 + 1), round(K^2/4 + K/4) , round(3*K^2 / 4 - K/4  ),...
    round(K^2 / 2 - K/4 + 1 ), round(K^2 / 2 + K/4 )  ] ;


%Reader position for monostatic and bistatic system
u_SDRReader= grid_2D(:,idx_pos_SDRReader);
u_CE = grid_2D(:,idx_pos_CE(1:L));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%       random experiment monte Carlo parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%create the eligible index positions for tags in bistatic and monostatic
%systems
idx_pos_mon = setdiff(1:K^2, [idx_pos_SDRReader]);
idx_pos_bis = setdiff(1:K^2, [idx_pos_SDRReader, idx_pos_CE]);

%theshold theta
theta_harvest_dB = [-30:5:-10] ;

%maximum number of iteration
MaxItersMonteCarlo = 1000:-20:100; % MaxItersMonteCarlo = 5000:-50:100;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%tables of outage probabilities
avg_energy_outage_thoery_mon = zeros(size(theta_harvest_dB));
avg_energy_outage_thoery_bis =  zeros(size(theta_harvest_dB));
max_energy_outage_thoery_mon = zeros(size(theta_harvest_dB));
max_energy_outage_thoery_bis =  zeros(size(theta_harvest_dB));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for theta_iter = 1 : length(theta_harvest_dB)
    
    theta_iter
    
    %current value of theta in normal scale
    theta_harvest_cur = 10^(theta_harvest_dB(theta_iter)/10);
    
    for iter_out = 1 : MaxItersMonteCarlo(theta_iter)
        
        
        %% small-scale fading generation
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%  Nakagami parameters for monostatic
        M_n = 1 +4*rand(N,1);
        %%%%%%%%%%%%%%%%%%%
        
        %%% Nakagami parameters for Bistatic
        M_ln = 1 + 4*rand(N,L);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        %% PLEs generation
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Path-loss exponents for monostatic
        PLE_mon_TR = 2 + 0.5*rand(N,1);
        
        % Path-loss exponents for bistatic
        PLE_bis_CT = 2 + 0.5*rand(N,L);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        
        
        %% create random topology for the experiment
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %choose  a network configuration randomly for monostatic system
        ids_pos_rnd_TAGS_mon = datasample(idx_pos_mon, N, 'Replace',false);
        
        %tag positions for the current experiment for monostatic system
        u_TAGS_mon = grid_2D(:, ids_pos_rnd_TAGS_mon);
        
        %choose  a network configuration randomly for bistatic system
        ids_pos_rnd_TAGS_bis = datasample(idx_pos_bis, N, 'Replace',false);
        
        %tag positions for the current experiment for bistatic system
        u_TAGS_bis = grid_2D(:, ids_pos_rnd_TAGS_bis);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        %% create experiment for  monostatic system
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %distance between tags and SDR reader
        d_TR_all_mon = pdist2(u_TAGS_mon', u_SDRReader');
        
        %one-way path gain for TR link for monostatic system
        L_TR_all_mon = ((lambda./(4*pi*d_ref)).^2).*((d_ref./d_TR_all_mon).^PLE_mon_TR);
        
        %average probability of energy outage for monostatic system
        avg_energy_outage_thoery_mon(theta_iter) = avg_energy_outage_thoery_mon(theta_iter) +...
            mean(((gammainc(theta_harvest_cur*M_n./(L_TR_all_mon*SDRReader_Tx_power_Watts ),M_n).*gamma(M_n)) ./ gamma(M_n)).^L);
        
        %maximum probability of energy outage for monostatic system
        max_energy_outage_thoery_mon(theta_iter) = max_energy_outage_thoery_mon(theta_iter) +...
            max(((gammainc(theta_harvest_cur*M_n./(L_TR_all_mon*SDRReader_Tx_power_Watts ),M_n).*gamma(M_n)) ./ gamma(M_n)).^L);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        %% create experiment for  bistatic system
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %distance between tags and carrier emitters
        d_CT_all_bis = pdist2(u_TAGS_bis', u_CE');
        
        
        %one-way path gain for CT link for bistatic system
        L_CT_all_bis = ((lambda./(4*pi*d_ref)).^2).*((d_ref./d_CT_all_bis).^PLE_bis_CT) ;
        
        
        %average probability of outage for bistatic system
        avg_energy_outage_thoery_bis(theta_iter) =  avg_energy_outage_thoery_bis(theta_iter)+...
            mean( prod((gammainc(theta_harvest_cur*M_ln./(L_CT_all_bis*CE_Tx_power_Watts ),M_ln).*gamma(M_ln)) ./ gamma(M_ln), 2) );
        
        %maximum probability of outage for bistatic system
        max_energy_outage_thoery_bis(theta_iter) = max_energy_outage_thoery_bis(theta_iter) +...
            max( prod((gammainc(theta_harvest_cur*M_ln./(L_CT_all_bis*CE_Tx_power_Watts ),M_ln).*gamma(M_ln)) ./ gamma(M_ln), 2) );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
    end
    %final outage proability
    avg_energy_outage_thoery_mon(theta_iter)  = avg_energy_outage_thoery_mon(theta_iter) /MaxItersMonteCarlo(theta_iter);
    avg_energy_outage_thoery_bis(theta_iter) =avg_energy_outage_thoery_bis(theta_iter) /MaxItersMonteCarlo(theta_iter);
    max_energy_outage_thoery_mon(theta_iter) = max_energy_outage_thoery_mon(theta_iter)/MaxItersMonteCarlo(theta_iter);
    max_energy_outage_thoery_bis(theta_iter) = max_energy_outage_thoery_bis(theta_iter)/MaxItersMonteCarlo(theta_iter);
    
end

avg_energy_outage_thoery_mon
avg_energy_outage_thoery_bis
max_energy_outage_thoery_mon
max_energy_outage_thoery_bis
 


