%%  Author: Panos N. Alevizos
%
%   Ph.D. Student and Research Assistant
%   School of Electrical and Computer Engineering
%   Technical University of Crete
%   Email: palevizos@isc.tuc.gr or bigpan88@hotmail.com
%   Webpage: http://users.isc.tuc.gr/~palevizos/
%
%   Date: January 2018
%
%   References:
%   P. N. Alevizos, K. Tountas, and A. Bletsas, "Multistatic Scatter Radio Sensor Networks for Extended Coverage," IEEE Transactions On Wireless Communications,
%   Volume: XX, Issue: XX, pp. XX--XX, 2018.
%
%   **Inquiries regarding the script provided below are cordially welcome.
%   In case you spot a bug, please let me know.
%   If you use some piece of code for your own work, please cite the
%   article above.**
%
%% this code is for Fig. 8 of the paper

close all;
clear all;
clc;





%%   Propagation and channel variance and noise Parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%noise figure (dB)
noiseFigure = 5;
%noise energy (dBm/Hz)
N0_dB = -174 + noiseFigure;
%Noise energy (Watt/Hz)
N0 = 0.001*10^(N0_dB/10);
%F carier
F_carrier = 0.868 * 10^9;
%propagation wavelength (meters)
lambda = 3*10^8 / F_carrier;
%reference distance
d_ref = 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





%%              Tag Parameters.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Tag data rate (bps).
rate = 1e3;
%bit duration
T = 1/rate;
%Tag scattering efficiency.
scatter_eff = 1e-1;
%antenna load values
Gamma_0 = 1;
Gamma_1 = -1;
Gamma_0_1 = abs(Gamma_0 - Gamma_1);
%number of tags 
N = 5 % N = 100

%frequency spacing
F_spacing = 10000;

%generate frequencies
Freqs_tags = zeros(2,N);
Freqs_tags(1,:) = 100000 :  F_spacing : 100000 + (N-1)*F_spacing;
Freqs_tags(2,:) = 100000 + F_spacing/5 :  F_spacing : 100000 + F_spacing/5 + (N-1)*F_spacing;

% Modulation index (BFSK modulation).
nu =2;
%create tag-subcarier frequency minimum seperation
rho_nj = zeros(N);
rho_nj_new = zeros(N);
for i = 1 : N
    for j = 1 : N
        if j ~= i
            rho_nj(i,j) =  (2*pi*T)^(-nu) * ...
                max(abs([(Freqs_tags(1,i) - Freqs_tags(1,j)).^(-nu),...
                (Freqs_tags(1,i) - Freqs_tags(2,j)).^(-nu),...
                (Freqs_tags(2,i) - Freqs_tags(1,j)).^(-nu),...
                (Freqs_tags(2,i) - Freqs_tags(2,j)).^(-nu)]));
%             rho_nj_new(i,j)  = 25./((2*pi*T*(5*abs(i-j)-1)* F_spacing)^2);
        end
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%             Emitter Parameters.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%number of emitters (and slots)
L =  4;
% Emitter transmit power (Watts).
CE_Tx_power_Watts = 0.001 * 10^(28/10)       
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%             SDR reader Parameters.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SDR reader transmit power (Watts).
SDRReader_Tx_power_Watts =  0.001* 10^(28/10);     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%     topology parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Grid parameters.
M = 200;                  % axis length (m).
Delta = 5;               % Grid resolution (m).
K = round(M/Delta) + 1;     %number of grid points per dimention

%grid 2 x 2
grid_2D = zeros(2, K^2);
grid_2D(1,:) = kron(ones(1,K), [0:K-1])*Delta;
grid_2D(2,:) = kron([0:K-1], ones(1,K))*Delta;


%indexes of position with respect to grid_2D matrix
%SDR reader position index
idx_pos_SDRReader = round(K^2/2);
%carrier emitters position index
idx_pos_CE= [ round(K^2/4+1) , round(K^2/4 + K/2), round(3*K^2 / 4 ), ...
    round(3*K^2 / 4 - K/2 + 1), round(K^2/4 + K/4) , round(3*K^2 / 4 - K/4  ),...
    round(K^2 / 2 - K/4 + 1 ), round(K^2 / 2 + K/4 )  ] ;

%Reader position for monostatic and bistatic system
u_SDRReader= grid_2D(:,idx_pos_SDRReader);
u_CE = grid_2D(:,idx_pos_CE(1:L));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





%%       random experiment monte Carlo parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%create the eligible index positions for tags in bistatic and monostatic
%systems
idx_pos_mon = setdiff(1:K^2, [idx_pos_SDRReader]);
idx_pos_bis = setdiff(1:K^2, [idx_pos_SDRReader, idx_pos_CE]);

%theshold theta
theta_dB = [-10 : 5 : 20] ;

%maximum number of iterations
MaxItersMonteCarlo = 200; % MaxItersMonteCarlo = 2000;

%number of channel realizations
channelRealizationIters = 3000;

%received SINR for monostatic and bistatic systems
SINR_inst_mon = zeros(N*L, channelRealizationIters);
SINR_inst_bis = SINR_inst_mon;


%tables of outage probabilities
outage_monostatic_simulation = zeros(size(theta_dB));
outage_bistatic_simulation = outage_monostatic_simulation;
 

%auxhilary data structure that precomputed for memory and speed efficiency
not_n = zeros(N-1,N);
for n = 1 : N
    not_n(:,n) = setdiff(1:N, n)';
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%outage per user after L slots for both systems
outage_per_user_for_L_slots_mon = zeros(N, channelRealizationIters);
outage_per_user_for_L_slots_bis = outage_per_user_for_L_slots_mon;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for theta_iter = 1 : length(theta_dB)
    
    theta_iter
    
    %current value of theta in normal scale
    theta_cur = 10^(theta_dB(theta_iter)/10);
    
    for iter_out = 1 : MaxItersMonteCarlo
        
        
        %% small-scale fading generation
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %small-scale fading for monostatic system
        M_n = 1 +4*rand(N,1);
        M_n_kron = kron(ones(1, channelRealizationIters), M_n);
        gamma_monostatic_alpha_param = kron(ones(L,channelRealizationIters), M_n);
        gamma_monostatic_beta_param = 1./gamma_monostatic_alpha_param;
        g_mon = gamrnd(gamma_monostatic_alpha_param ,gamma_monostatic_beta_param  ).^2;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %small-scale fading for bistatic system
        M_ln = 1 + 4*rand(N*L,1);
        gamma_bistatic_CT_alpha_param = kron(ones(1,channelRealizationIters), M_ln);
        gamma_bistatic_CT_beta_param = 1./gamma_bistatic_CT_alpha_param; 
        gamma_bistatic_TR_alpha_param = gamma_monostatic_alpha_param;
        gamma_bistatic_TR_beta_param =  gamma_monostatic_beta_param;
        g_bis = gamrnd(gamma_bistatic_CT_alpha_param ,gamma_bistatic_CT_beta_param  ).* gamrnd(gamma_bistatic_TR_alpha_param ,gamma_bistatic_TR_beta_param );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        

        %% PLEs generation
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Path-loss exponents for monostatic
        PLE_mon_TR = 2 + 0.5*rand(N,1);
        
        % Path-loss exponents for bistatic
        PLE_bis_TR = PLE_mon_TR;
        PLE_bis_CT = 2 + 0.5*rand(N,L);
        PLE_bis_TR_vec = kron(ones(L,1),PLE_bis_TR);
        PLE_bis_CT_vec = PLE_bis_CT(:);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        
        %% create random channel assignment for all tags
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        rand_chann_assign = randperm(N);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        %% create random topology for the experiment
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %choose  a network configuration randomly for monostatic system
        ids_pos_rnd_TAGS_mon = datasample(idx_pos_mon, N, 'Replace',false);
        
        %tag positions for the current experiment for monostatic system
        u_TAGS_mon = grid_2D(:, ids_pos_rnd_TAGS_mon);
        
        %choose  a network configuration randomly for bistatic system
        ids_pos_rnd_TAGS_bis = datasample(idx_pos_bis, N, 'Replace',false);
        
        %tag positions for the current experiment for bistatic system
        u_TAGS_bis = grid_2D(:, ids_pos_rnd_TAGS_bis);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
          %% create experiment for  monostatic system
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %distance between tags and SDR reader
        d_TR_all_mon = pdist2(u_TAGS_mon', u_SDRReader');
        
        %one-way path gain for TR link for monostatic system
        L_TR_all_mon = ((lambda./(4*pi*d_ref)).^2).*((d_ref./d_TR_all_mon).^PLE_mon_TR);


        %\mu for monostatic (Eq. 15)
        mu_mon = sqrt(2 * SDRReader_Tx_power_Watts) * kron(L_TR_all_mon, ...
            ones(1, channelRealizationIters)) * Gamma_0_1 * (2/pi) * scatter_eff ;
        %Average received energy for monostatic (Eq. 14)
        E_mon = T * (mu_mon).^2;
        
        %instantaneous SINR for monostatic
        for l = 1 : L
            for n = 1 : N
                
                %random assignemnt of tag n
                n_perm = rand_chann_assign(n);
                rho_nj_realization = kron(rho_nj(not_n(:,n_perm), n_perm), ones(1, channelRealizationIters));
                
                %instantaneous SINR (Eq. 23)
                SINR_inst_mon((l-1)*N + n,:) = E_mon(n,:).* (M_n(n)./(M_n(n) + 1)).* g_mon((l-1)*N + n,:) ./ ...
                    (sum(E_mon(not_n(:,n),:).* (M_n_kron(not_n(:,n),:)./(M_n_kron(not_n(:,n),:) + 1)).* rho_nj_realization.*g_mon((l-1)*N + not_n(:,n),:)) +  N0) ;
            end 
        end
        
        %hard coded realization: 1 --> outage, 0 --> non-outage
        realizations = SINR_inst_mon <=  theta_cur;
        %intersection over all time slots
        for n = 1 : N
            outage_per_user_for_L_slots_mon(n,:) = prod(realizations(n:N:end, :));
        end
        %event of information outage for monostatic system
        outage_monostatic_simulation(theta_iter) = outage_monostatic_simulation(theta_iter) ...
            + mean( mean(outage_per_user_for_L_slots_mon >= 1) );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        
        %% create experiment for  bistatic system
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %distance between tags and carrier emitters
        d_CT_all_bis = pdist2(u_TAGS_bis', u_CE');
        d_CT_all_bis = d_CT_all_bis(:);
        
        %distance between tags and SDR reader
        d_TR_all_bis = pdist2(u_TAGS_bis', u_SDRReader');
        d_TR_all_bis = kron(ones(L,1), d_TR_all_bis);
        
        %one-way path gain for CT link for bistatic system
        L_CT_all_bis = ((lambda./(4*pi*d_ref)).^2).*((d_ref./d_CT_all_bis).^PLE_bis_CT_vec) ;
        
        %one-way path gain for TR link for bistatic system
        L_TR_all_bis = ((lambda./(4*pi*d_ref)).^2).*((d_ref./d_TR_all_bis).^PLE_bis_TR_vec);
           
        
        %\mu for bistatic (Eq. 10)
        mu_bis = sqrt(2* CE_Tx_power_Watts) * kron(sqrt(L_CT_all_bis.*L_TR_all_bis),...
            ones(1, channelRealizationIters)) * Gamma_0_1 * (2/pi) * scatter_eff;
        %Average received energy for monostatic (Eq. 9)
        E_bis = 0.5 * T * (mu_bis).^2;
        
        %instantaneous SINR for bistatic system
        for l = 1 : L
            for n = 1 : N
                
                %random assignemnt of tag n
                n_perm = rand_chann_assign(n);
                rho_nj_realization = kron(rho_nj(not_n(:,n_perm), n_perm), ones(1, channelRealizationIters));
                
                %instantaneous SINR (Eq. 27)
                SINR_inst_bis((l-1)*N + n,:) = E_bis((l-1)*N + n,:).*g_bis((l-1)*N + n,:) ./ ...
                    (sum(E_bis((l-1)*N + not_n(:,n),:).*rho_nj_realization.*g_bis((l-1)*N + not_n(:,n),:)) +  N0) ;
                
            end
        end
        
        %hard coded realization: 1 --> outage, 0 --> non-outage
        realizations = SINR_inst_bis <=  theta_cur;
        %intersection over all time slots
        for n = 1 : N
            outage_per_user_for_L_slots_bis(n,:) = prod(realizations(n:N:end, :));
        end
        %event of information outage for bistatic system
        outage_bistatic_simulation(theta_iter) = outage_bistatic_simulation(theta_iter) ...
            + mean( mean(outage_per_user_for_L_slots_bis >= 1) );
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    end
    %final outage proability
    outage_monostatic_simulation(theta_iter) = outage_monostatic_simulation(theta_iter)/MaxItersMonteCarlo;
    outage_bistatic_simulation(theta_iter) = outage_bistatic_simulation(theta_iter)/MaxItersMonteCarlo;
    
end

 outage_monostatic_simulation
 outage_bistatic_simulation

%%
semilogy( theta_dB, outage_monostatic_simulation, 'b-',...
    theta_dB, outage_bistatic_simulation, '--r')
xlim([theta_dB(1), theta_dB(end)])


